########################################################
# FUNCTIONS NEEDED TO ESTIMATE MORATLITY AND INCIDENCE
########################################################

############################
# Calculation of the rates, given PREVALENCES
#   
# from total survival = weighted survival of all groups
#    i.e: exp(-m)= \sum_k p_k exp(-m_k)
#    and mortality rate of group k:   m_k = r_k*m_nSnD
#    for given relative risk r_k
# Thus, find root of: m + log(\sum_k p_k exp(-r_k*m_nSnD))
###################################################################
calratesS.fun <-function(mnSnD,prevs,relrisks,mtot) {
  S = sum(prevs*exp(-c(relrisks*mnSnD,mnSnD)))    
  f = mtot+log(S)
  return(f)
}

calratesS.dfun <-function(mnSnD,prevs,relrisks,mtot) {
  rr = c(relrisks,1)
  S = sum(prevs*exp(-c(relrisks*mnSnD,mnSnD)))
  A = sum(prevs*rr*exp(-c(relrisks*mnSnD,mnSnD)))
  return(-A/S)
}       
start.rates <-function(prevs,relrisks,mtot) {
  return(mtot/
         (prevs[1]*relrisks[1]+prevs[2]*relrisks[2]+
         prevs[3]*relrisks[3] + prevs[4]))
}

start.rates2 <-function(prevs,relrisks,mtot) {
  return(mtot/
         (prevs[1]*relrisks + prevs[2]))
}
################################################################
#  ESTIMATION OF INCIDENCE, GIVEN MORTALITY
#   (separately for smoking and non-smoking, phi_S=0)
#
# from DISABILITY MODEL, i.e. (age-year specific) Markov model 
# with rate matrix  (conditional on smoking status)
#
#          Dis         non-Dis   death
#  M = -(phi+m_D)       phi      m_D
#         theta    -(theta+m_N)  m_N
#          0            0         0
#
#  with phi, recoveray rate (=0!)
#       theta, incidence rate 
#   
#   This lead to (some complicated formulas)
#            transition probabilities at age x to x+1
#       P_DD(x,x+1), P_DN(x,x+1), P_ND(x,x+1) and P_NN(x,x+1)
#   Then the relative number (of smokers) people disabled 
#         N_D^S(x+1) = p_SD(x)*P_DD(x,x+1)+p_SnD(x)*P_ND(x,x+1)
#   and relative number of non-disabled people
#         N_nD^S(x+1) = p_SD(x)*P_DN(x,x+1)+p_SnD(x)*P_NN(x,x+1) 
#   Then the proportion of (smokers) people of age x+1 disabled is:
#   
#        p_SD(x+1)                   N_D^S(x+1)
#    ---------------       =   -----------------------
#    p_SD(x+1)+p_SnD(x+1)       N_D^S(x+1)+N_nD^S(x+1)     
#
#  This will be solved w.r.t. theta
#     (similar for non-smokers)
###################################################################
transprob = function(theta, phi, mD, mnD) {
  # eigenvalues of rate matrix M, lambda1=0, not relevant
  lambda2 = (-0.5*(phi+mD+theta+mnD)
             -0.5*sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  lambda3 = (-0.5*(phi+mD+theta+mnD)
             +0.5*sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  P_DD = ((theta+mnD+lambda2)*exp(lambda2) 
           - (theta+mnD+lambda3)*exp(lambda3))/(lambda2-lambda3)
  P_NN = ((phi+mD+lambda2)*exp(lambda2) 
           - (phi+mD+lambda3)*exp(lambda3))/(lambda2-lambda3)
  P_DN = phi*(exp(lambda2)- exp(lambda3))/(lambda2-lambda3)
  P_ND = theta*(exp(lambda2)- exp(lambda3))/(lambda2-lambda3)
  P = matrix(c(P_DD,P_ND,P_DN,P_NN), nrow=2)
  rownames(P) = c("D", "nD")
  colnames(P) = c("D", "nD")
  return(P)
}

dtransprob = function(theta, phi, mD, mnD) {
  # derivative w.r.t. theta
  # eigenvalues of rate matrix M, lambda1=0, not relevant
  lambda2 = (-0.5*(phi+mD+theta+mnD)
             -0.5*sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  lambda3 = (-0.5*(phi+mD+theta+mnD)
             +0.5*sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  dlambda2 = -0.5*(1+(phi-mD+theta+mnD)
                     /sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))          
  dlambda3 = -0.5*(1-(phi-mD+theta+mnD)
                     /sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta)) 
  dlambda23 =  -((phi-mD+theta+mnD)
                     /sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  A1a = ((theta+mnD+lambda2)*exp(lambda2)- (theta+mnD+lambda3)*exp(lambda3))
  A1b = ((phi+mD+lambda2)*exp(lambda2)- (phi+mD+lambda3)*exp(lambda3))
  A2 = (exp(lambda2)- exp(lambda3))
  dP_DD = (-dlambda23*A1a/((lambda2-lambda3)^2) +
            ( (1+dlambda2*(1+theta+mnD+lambda2))*exp(lambda2) -
               (1+dlambda3*(1+theta+mnD+lambda3))*exp(lambda3))/(lambda2-lambda3) )
  dP_NN = (-dlambda23*A1b/((lambda2-lambda3)^2) +
            ( dlambda2*(1+phi+mD+lambda2)*exp(lambda2) -
               dlambda3*(1+phi+mD+lambda3)*exp(lambda3))/(lambda2-lambda3)   )
  dP_DN = phi*(-dlambda23*A2/((lambda2-lambda3)^2) +
               ( dlambda2*exp(lambda2) - dlambda3*exp(lambda3))/(lambda2-lambda3)   )
  dP_ND = ( A2/(lambda2-lambda3)+
            theta*(  -dlambda23*A2/((lambda2-lambda3)^2) +
                     ( dlambda2*exp(lambda2) - dlambda3*exp(lambda3))
                      /(lambda2-lambda3)   )                           )
  dP = matrix(c(dP_DD,dP_ND,dP_DN,dP_NN), nrow=2)
  rownames(dP) = c("D", "nD")
  colnames(dP) = c("D", "nD")
  return(dP)
}


#### fun.inc is the function for which we want the root, theta
# input:
#   propD : proportion disabled at age x (given year and smoking status)
#   propD1: proportion disabled at age x+1 (given year and smoking status)
#   mDis  :  mortality rate of disabled at age x 
#                  (given year and smoking status)
#   mnDis :  mortality rate of non-disabled at age x 
#                  (given year and smoking status)
#   phi: recovery rate (default = 0)
  
fun.inc = function(theta, propD, propD1, mDis,mnDis, phi=0) {
  Ptrans = transprob(theta, phi, mDis, mnDis)
  NDis = propD*Ptrans[1,1]+(1-propD)*Ptrans[2,1]
  NnDis = propD*Ptrans[1,2]+(1-propD)*Ptrans[2,2]
  return(NDis*(1-propD1)-propD1*NnDis)
}

dfun.inc = function(theta, propD, propD1, mDis, mnDis, phi=0) {
  dPtrans = dtransprob(theta, phi, mDis, mnDis)
  dNDis = propD*dPtrans[1,1]+(1-propD)*dPtrans[2,1]
  dNnDis = propD*dPtrans[1,2]+(1-propD)*dPtrans[2,2]
  return(dNDis*(1-propD1)-propD1*dNnDis)
}
start.inc.rates <-function(propnD, propnD1) {
  return(log(propnD/propnD1))
}
################################################################
#  ESTIMATION OF MORTALITY GIVEN INCIDENCE and RECOVERY
################################################################
surv.inc2.O = function(m, theta, prop0, relrisk, mtot, phi=0) {
  mDisO = relrisk[1]*m
  mnDisO = relrisk[2]*m
  mDisnO = relrisk[3]*m
  PtransO = transprob(theta["theta_O"], phi, mDisO, mnDisO)
  PtransnO = transprob(theta["theta_nO"], phi, mDisnO, m)
  Surv.est = prop0["OD"]*(PtransO[1,1]+PtransO[1,2]) +
             prop0["OnD"]*(PtransO[2,1]+PtransO[2,2]) +
             prop0["nOD"]*(PtransnO[1,1]+PtransnO[1,2]) +
             prop0["nOnD"]*(PtransnO[2,1]+PtransnO[2,2]) 
  return((Surv.est-exp(-mtot))^2)
}

############################################################
# TOTAL TRANSITION PROBABILITY
############################################################

## WAAROM WIJKT DEZE EEN BEETJE AF VAN DE 2 LOSSE????

Transprob.tot<- function(TransRates,agestart=55, ageend=89) {
 
 n.statestot = dim(TransRates)[2]
 subnames = dimnames(TransRates)[[2]]
 n.states = n.statestot-1
 ageint = agestart:ageend
 n.age = ageend-agestart+1
 
 TransProb = array(0,dim=c((n.states+1),(n.states+1),n.age))
 colnames(TransProb) = c("OD","OnD","nOD","nOnD","dead")
 rownames(TransProb) = c("OD","OnD","nOD","nOnD","dead")
 dimnames(TP)[[3]] = ageint
 for (ai in 1:n.age) {
   Mmatrix = matrix(0,ncol=n.states+1,nrow=n.states+1)
   colnames(Mmatrix) = c("OD","OnD","nOD","nOnD","dead")
   rownames(Mmatrix) = c("OD","OnD","nOD","nOnD","dead")
   #colnames(Mmatrix) = c("nOnD","OnD","nOD","OD","dead")
   #rownames(Mmatrix) = c("nOnD","OnD","nOD","OD","dead")
   Mmatrix["nOnD", "nOD"]=-TransRates["nOnD", "nOD",ai]
   Mmatrix["nOnD", "dead"]=-TransRates["nOnD", "dead",ai]
   Mmatrix["nOD", "dead"]=-TransRates["nOD", "dead",ai]
   Mmatrix["OnD", "OD"]=-TransRates["OnD", "OD",ai]
   Mmatrix["OnD", "dead"]=-TransRates["OnD", "dead",ai]
   Mmatrix["OD", "dead"]=-TransRates["OD", "dead",ai]
   Mmatrix["nOnD", "nOnD"]=TransRates["nOnD", "nOD",ai]+
                           TransRates["nOnD", "dead",ai]
   Mmatrix["OnD", "OnD"]=TransRates["OnD", "OD",ai]+
                         TransRates["OnD", "dead",ai]
   Mmatrix["nOD", "nOD"]=TransRates["nOD", "dead",ai]
   Mmatrix["OD", "OD"]=TransRates["OD", "dead",ai]
   if (ai==20) {
     print(Mmatrix)
   }
   Mmatrix1=ifelse(is.nan(Mmatrix),0,Mmatrix) 
   EigM=eigen(Mmatrix1, symmetric=F)
   EigM$values=ifelse((EigM$values < 1e-12),0,EigM$values)
   U = EigM$vectors
   Uinv = solve(EigM$vectors)
   TransProb[,,ai] = t(U%*%diag(exp(-EigM$values))%*%Uinv)
 }
 return(TransProb)
}

Transprob.tot.ana<- function(TransRates,agestart=55, ageend=89) {
 
 n.statestot = dim(TransRates)[2]
 subnames = dimnames(TransRates)[[2]]
 n.states = n.statestot-1
 ageint = agestart:ageend
 n.age = ageend-agestart+1
 
 TransProb = array(0,dim=c((n.states+1),(n.states+1),n.age))
 colnames(TransProb) = c("OD","OnD","nOD","nOnD","dead")
 rownames(TransProb) = c("OD","OnD","nOD","nOnD","dead")
 dimnames(TP)[[3]] = ageint
 TransProb.ana = array(0,dim=c((n.states+1),(n.states+1),n.age))
 colnames(TransProb.ana) = c("OD","OnD","nOD","nOnD","dead")
 rownames(TransProb.ana) = c("OD","OnD","nOD","nOnD","dead")
 dimnames(TP)[[3]] = ageint
 Eigenval = matrix(0,nrow=n.age,ncol=(n.states+1))
 Eigenval.ana = matrix(0,nrow=n.age,ncol=(n.states+1))
 rownames(Eigenval) = ageint
 rownames(Eigenval.ana) = ageint
 Eigenvec = array(0,dim=c((n.states+1),(n.states+1),n.age))
 colnames(Eigenvec) = c("OD","OnD","nOD","nOnD","dead")
 rownames(Eigenvec) = c("OD","OnD","nOD","nOnD","dead")
 dimnames(Eigenvec)[[3]] = ageint
 Eigenvec.ana = array(0,dim=c((n.states+1),(n.states+1),n.age))
 colnames(Eigenvec.ana) = c("OD","OnD","nOD","nOnD","dead")
 rownames(Eigenvec.ana) = c("OD","OnD","nOD","nOnD","dead")
 dimnames(Eigenvec.ana)[[3]] = ageint
 Mmatrix.ana = array(0,dim=c((n.states+1),(n.states+1),n.age))
 colnames(Mmatrix.ana) = c("OD","OnD","nOD","nOnD","dead")
 rownames(Mmatrix.ana) = c("OD","OnD","nOD","nOnD","dead")
 dimnames(Mmatrix.ana)[[3]] = ageint
 for (ai in 1:n.age) {
   Mmatrix = matrix(0,ncol=n.states+1,nrow=n.states+1)
   colnames(Mmatrix) = c("OD","OnD","nOD","nOnD","dead")
   rownames(Mmatrix) = c("OD","OnD","nOD","nOnD","dead")
   #colnames(Mmatrix) = c("nOnD","OnD","nOD","OD","dead")
   #rownames(Mmatrix) = c("nOnD","OnD","nOD","OD","dead")
   Mmatrix["nOnD", "nOD"]=-TransRates["nOnD", "nOD",ai]
   Mmatrix["nOnD", "dead"]=-TransRates["nOnD", "dead",ai]
   Mmatrix["nOD", "dead"]=-TransRates["nOD", "dead",ai]
   Mmatrix["OnD", "OD"]=-TransRates["OnD", "OD",ai]
   Mmatrix["OnD", "dead"]=-TransRates["OnD", "dead",ai]
   Mmatrix["OD", "dead"]=-TransRates["OD", "dead",ai]
   Mmatrix["nOnD", "nOnD"]=TransRates["nOnD", "nOD",ai]+
                           TransRates["nOnD", "dead",ai]
   Mmatrix["OnD", "OnD"]=TransRates["OnD", "OD",ai]+
                         TransRates["OnD", "dead",ai]
   Mmatrix["nOD", "nOD"]=TransRates["nOD", "dead",ai]
   Mmatrix["OD", "OD"]=TransRates["OD", "dead",ai]
   Mmatrix1=ifelse(is.nan(Mmatrix),0,Mmatrix) 
   Mmatrix.ana[,,ai]=Mmatrix1
   EigM=eigen(Mmatrix1, symmetric=F)
   EigM$values=ifelse((EigM$values < 1e-12),0,EigM$values)
   Eigenval[ai,] = t(EigM$values)
   Eigenval.ana[ai,] = diag(Mmatrix1)
   U = EigM$vectors
   Uinv = solve(EigM$vectors)
   Eigenvec[,,ai] = Uinv
   Eigenvec.ana[,"OD",ai] = matrix(c(Mmatrix["OD", "OD"]-Mmatrix["OnD", "OnD"],
                                    Mmatrix["OnD", "OD"],0,0,0), ncol=1)
   Eigenvec.ana[,"OnD",ai] = matrix(c(0,1,0,0,0), ncol=1)
   Eigenvec.ana[,"nOD",ai] = matrix(c(0,0,Mmatrix["nOD", "nOD"]-Mmatrix["nOnD", "nOnD"],
                                           Mmatrix["nOnD", "nOD"],0), ncol=1)
   Eigenvec.ana[,"nOnD",ai] = matrix(c(0,0,0,1,0), ncol=1)
   Eigenvec.ana[,"dead",ai] = matrix(c(1,1,1,1,1), ncol=1)
   U.ana = Eigenvec.ana[,,ai]
   Uinv.ana = solve(U.ana)
   TransProb[,,ai] = t(U%*%diag(exp(-EigM$values))%*%Uinv)
   TransProb.ana[,,ai] = t(U.ana%*%diag(exp(-diag(Mmatrix1)))%*%Uinv.ana)
 }
 return(list(TP=TransProb,TP2=TransProb.ana, Matrix=Mmatrix.ana,
             Eig1=Eigenval, Eig2=Eigenval.ana, Eigvec1=Eigenvec , Eigvec2=Eigenvec.ana))
}

Transprob.eig<- function(theta, phi, mD, mnD) {
 
 TransProb = matrix(0,nrow=3, ncol=3)
 colnames(TransProb) = c("D","nD", "dead")
 rownames(TransProb) = c("D","nD", "dead")
 Mmatrix = matrix(0,ncol=3,nrow=3)
 colnames(Mmatrix) = c("D","nD", "dead")
 rownames(Mmatrix) = c("D","nD", "dead")
 Mmatrix["nD", "D"]=-theta
 Mmatrix["nD", "dead"]=-mnD
 Mmatrix["D", "dead"]=-mD
 Mmatrix["nD", "nD"]=(theta+mnD)
 Mmatrix["D", "D"]=mD
 #print(Mmatrix)
 EigM=eigen(Mmatrix, symmetric=F)
 EigM$values=ifelse((EigM$values < 1e-10),0,EigM$values)
 U = EigM$vectors
 Uinv = solve(EigM$vectors)
 TransProb = t(U%*%diag(exp(-EigM$values))%*%Uinv)
 return(TransProb)
}

############################################################
# SIMULTANE SCHATTING
############################################################
make.transmat.O = function(th,mort){
  # th = (theta_D^nO, theta_D^S), disability incidence rates
  # mort=(m_SD, m_SnD, m_nSD, m_nSnD) mortality rates
  
  transmat1 = cbind(-(mort["OD"]), 0, 0, 0, mort["OD"])
  transmat2 = cbind(th["theta_O"], -(th["theta_O"]+mort["OnD"]), 0, 0, mort["OnD"])
  transmat3 = cbind(0,0,-(mort["nOD"]), 0, mort["nOD"])
  transmat4 = cbind(0,0,th["theta_nO"],-(th["theta_nO"]+mort["nOnD"]), mort["nOnD"])
  transmat = rbind(transmat1,transmat2, transmat3, transmat4, rep(0,5))
  colnames(transmat) = c("OD", "OnD", "nOD", "nOnD", "death")
  rownames(transmat) = c("OD", "OnD", "nOD", "nOnD", "death")
  return(transmat)
}

est.Prev1.Surv.O = function(Prev0,th,mort){
   rate.mat = -make.transmat.O( th,mort)
   Eigvecval = eigen(rate.mat, symmetric=F)
   Eigvecval$values=ifelse((Eigvecval$values < 1e-12),0,Eigvecval$values)
   U =  Eigvecval$vectors
   Uinv = solve(Eigvecval$vectors)
   transprob = U%*%diag(exp(-Eigvecval$values))%*%Uinv
   colnames(transprob) = c("OD", "OnD", "nOD", "nOnD", "death")
   rownames(transprob) = c("OD", "OnD", "nOD", "nOnD", "death")
   print(transprob)
   print(Prev0)
   Prev.tmp = matrix(Prev0,ncol=4)%*%matrix(transprob[1:4,1:4],ncol=4)
   Prev1 = Prev.tmp/sum(Prev.tmp)
   names(Prev1) = c("OD", "OnD", "nOD", "nOnD")
   Surv1 = 1-(Prev0%*%transprob[1:4,5])
   return(list(Prev1=Prev1, Surv1=Surv1, transprob=transprob))
}

surv.O = function(m, th,Prev0, relrisk, mtot) {
  mort = c(relrisk["OD"]*m, relrisk["OnD"]*m, relrisk["nOD"]*m, m)
  names(mort)=c("OD", "OnD", "nOD", "nOnD")
  rate.mat = -make.transmat.O( th,mort)
  Eigvecval = eigen(rate.mat, symmetric=F)
  Eigvecval$values=ifelse((Eigvecval$values < 1e-12),0,Eigvecval$values)
  U =  Eigvecval$vectors
  Uinv = solve(Eigvecval$vectors)
  transprob = U%*%diag(exp(-Eigvecval$values))%*%Uinv
  colnames(transprob) = c("OD", "OnD", "nOD", "nOnD", "death")
  rownames(transprob) = c("OD", "OnD", "nOD", "nOnD", "death")
  Surv.est = 1-(matrix(Prev0,ncol=4)%*%matrix(transprob[1:4,5],nrow=4)) 
  return(1000*(Surv.est-exp(-mtot))^2)
}

est.Prev1.O = function(Prev0,th,mort){
   rate.mat = -make.transmat.O( th,mort)
  Eigvecval = eigen(rate.mat, symmetric=F)
  Eigvecval$values=ifelse((Eigvecval$values < 1e-12),0,Eigvecval$values)
  U =  Eigvecval$vectors
  Uinv = solve(Eigvecval$vectors)
  transprob = U%*%diag(exp(-Eigvecval$values))%*%Uinv
  colnames(transprob) = c("OD", "OnD", "nOD", "nOnD", "death")
  rownames(transprob) = c("OD", "OnD", "nOD", "nOnD", "death")
   Prev.tmp = matrix(Prev0,ncol=4)%*%matrix(transprob[1:4,1:4],ncol=4)
   Prev1 = Prev.tmp/sum(Prev.tmp)
   names(Prev1) = c("OD", "OnD", "nOD", "nOnD")
   return(Prev1)
}

quaddiff.O=function(pars,Prev0,Prev1,mort){
  # pars = (theta_nS, theta_S, b_S)
  # Q.est = est.Prev1(Prev0,exp(pars[1:2]),exp(pars[3]),mort,b_D)
   Q.est = est.Prev1.O(Prev0,pars,mort)
   f = sum((10000*Prev1[1:3]-10000*Q.est[1:3])^2) 
   return(f)
}
