######################################################################
#  CALCULATION OF (AGE-YEAR SPECIFIC) DISABILITY INCIDENCE RATES 
#                   AND MORTALITY RATES 
#------------------------------------------------------------------
# INPUT: - relative risks (based on ERGO)
#        - Total (age-specific) mortality rates(EUROPOP)
#                 Mort_NL_M.csv or Mort_NL_F.csv
#        -(age-specific) ADL-prevalence (based on SHARE)
#                 Prev_ADL_NL_M.csv or Mort_NL_F.csv
#
#   Choose gender: M or F
#
# OUTPUT:
#    -(age-specific) ADL-incidence rates
#                 Dis_inc_NL_M.txt or Dis_inc_NL_F.txt
#        - (age-specific) ADL-specific (disabled/non-disabled) mortality rates
#                 Mort_ADL_NL_M.txt or Mort_ADL_NL_F.txt
########################################################################
rm(list=ls())

############################################################
#                           FUNCTIONS
###########################################################
calratesS.fun <-function(mnSnD,prevs,relrisks,mtot) {
  S = sum(prevs*exp(-c(relrisks*mnSnD,mnSnD)))    
  f = mtot+log(S)
  return(f)
}

calratesS.dfun <-function(mnSnD,prevs,relrisks,mtot) {
  rr = c(relrisks,1)
  S = sum(prevs*exp(-c(relrisks*mnSnD,mnSnD)))
  A = sum(prevs*rr*exp(-c(relrisks*mnSnD,mnSnD)))
  return(-A/S)
}       
start.rates2 <-function(prevs,relrisks,mtot) {
  return(mtot/
         (prevs[1]*relrisks + prevs[2]))
}
#------------------------------------------------------#
################################################################
#  ESTIMATION OF INCIDENCE, GIVEN MORTALITY
#   (separately for smoking and non-smoking, phi_S=0)
#
# from DISABILITY MODEL, i.e. (age-year specific) Markov model 
# with rate matrix  (conditional on smoking status)
#
#          Dis         non-Dis   death
#  M = -(phi+m_D)       phi      m_D
#         theta    -(theta+m_N)  m_N
#          0            0         0
#
#  with phi, recoveray rate (=0!)
#       theta, incidence rate 
#   
#   This lead to (some complicated formulas)
#            transition probabilities at age x to x+1
#       P_DD(x,x+1), P_DN(x,x+1), P_ND(x,x+1) and P_NN(x,x+1)
#   Then the relative number (of smokers) people disabled 
#         N_D^S(x+1) = p_SD(x)*P_DD(x,x+1)+p_SnD(x)*P_ND(x,x+1)
#   and relative number of non-disabled people
#         N_nD^S(x+1) = p_SD(x)*P_DN(x,x+1)+p_SnD(x)*P_NN(x,x+1) 
#   Then the proportion of (smokers) people of age x+1 disabled is:
#   
#        p_SD(x+1)                   N_D^S(x+1)
#    ---------------       =   -----------------------
#    p_SD(x+1)+p_SnD(x+1)       N_D^S(x+1)+N_nD^S(x+1)     
#
#  This will be solved w.r.t. theta
#     (similar for non-smokers)
###################################################################
transprob = function(theta, phi, mD, mnD) {
  # eigenvalues of rate matrix M, lambda1=0, not relevant
  lambda2 = (-0.5*(phi+mD+theta+mnD)
             -0.5*sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  lambda3 = (-0.5*(phi+mD+theta+mnD)
             +0.5*sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  P_DD = ((theta+mnD+lambda2)*exp(lambda2) 
           - (theta+mnD+lambda3)*exp(lambda3))/(lambda2-lambda3)
  P_NN = ((phi+mD+lambda2)*exp(lambda2) 
           - (phi+mD+lambda3)*exp(lambda3))/(lambda2-lambda3)
  P_DN = phi*(exp(lambda2)- exp(lambda3))/(lambda2-lambda3)
  P_ND = theta*(exp(lambda2)- exp(lambda3))/(lambda2-lambda3)
  P = matrix(c(P_DD,P_ND,P_DN,P_NN), nrow=2)
  rownames(P) = c("D", "nD")
  colnames(P) = c("D", "nD")
  return(P)
}

dtransprob = function(theta, phi, mD, mnD) {
  # derivative w.r.t. theta
  # eigenvalues of rate matrix M, lambda1=0, not relevant
  lambda2 = (-0.5*(phi+mD+theta+mnD)
             -0.5*sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  lambda3 = (-0.5*(phi+mD+theta+mnD)
             +0.5*sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  dlambda2 = -0.5*(1+(phi-mD+theta+mnD)
                     /sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))          
  dlambda3 = -0.5*(1-(phi-mD+theta+mnD)
                     /sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta)) 
  dlambda23 =  -((phi-mD+theta+mnD)
                     /sqrt(((phi+mD-theta-mnD)^2) + 4*phi*theta))
  A1a = ((theta+mnD+lambda2)*exp(lambda2)- (theta+mnD+lambda3)*exp(lambda3))
  A1b = ((phi+mD+lambda2)*exp(lambda2)- (phi+mD+lambda3)*exp(lambda3))
  A2 = (exp(lambda2)- exp(lambda3))
  dP_DD = (-dlambda23*A1a/((lambda2-lambda3)^2) +
            ( (1+dlambda2*(1+theta+mnD+lambda2))*exp(lambda2) -
               (1+dlambda3*(1+theta+mnD+lambda3))*exp(lambda3))/(lambda2-lambda3) )
  dP_NN = (-dlambda23*A1b/((lambda2-lambda3)^2) +
            ( dlambda2*(1+phi+mD+lambda2)*exp(lambda2) -
               dlambda3*(1+phi+mD+lambda3)*exp(lambda3))/(lambda2-lambda3)   )
  dP_DN = phi*(-dlambda23*A2/((lambda2-lambda3)^2) +
               ( dlambda2*exp(lambda2) - dlambda3*exp(lambda3))/(lambda2-lambda3)   )
  dP_ND = ( A2/(lambda2-lambda3)+
            theta*(  -dlambda23*A2/((lambda2-lambda3)^2) +
                     ( dlambda2*exp(lambda2) - dlambda3*exp(lambda3))
                      /(lambda2-lambda3)   )                           )
  dP = matrix(c(dP_DD,dP_ND,dP_DN,dP_NN), nrow=2)
  rownames(dP) = c("D", "nD")
  colnames(dP) = c("D", "nD")
  return(dP)
}


#### fun.inc is the function for which we want the root, theta
# input:
#   propD : proportion disabled at age x (given year and smoking status)
#   propD1: proportion disabled at age x+1 (given year and smoking status)
#   mDis  :  mortality rate of disabled at age x 
#                  (given year and smoking status)
#   mnDis :  mortality rate of non-disabled at age x 
#                  (given year and smoking status)
#   phi: recovery rate (default = 0)
  
fun.inc = function(theta, propD, propD1, mDis,mnDis, phi=0) {
  Ptrans = transprob(theta, phi, mDis, mnDis)
  NDis = propD*Ptrans[1,1]+(1-propD)*Ptrans[2,1]
  NnDis = propD*Ptrans[1,2]+(1-propD)*Ptrans[2,2]
  return(NDis*(1-propD1)-propD1*NnDis)
}

dfun.inc = function(theta, propD, propD1, mDis, mnDis, phi=0) {
  dPtrans = dtransprob(theta, phi, mDis, mnDis)
  dNDis = propD*dPtrans[1,1]+(1-propD)*dPtrans[2,1]
  dNnDis = propD*dPtrans[1,2]+(1-propD)*dPtrans[2,2]
  return(dNDis*(1-propD1)-propD1*dNnDis)
}
start.inc.rates <-function(propnD, propnD1,smok) {
  return(log(propnD/propnD1))
}
surv.inc.adl = function(m, theta, prop0, relrisk, mtot, phi=0) {
  mDis = relrisk*m
  Ptrans = transprob(theta, phi, mDis, m)
  Surv.est = prop0["D"]*(Ptrans[1,1]+Ptrans[1,2]) +
             prop0["nD"]*(Ptrans[2,1]+Ptrans[2,2]) 
  return((Surv.est-exp(-mtot))^2)
}
###################################################################
#              INPUT
###################################################################

gender = "F"          # Either M(ale) or F(emale)     

age.start=55
age.end = 100
n.age = age.end-age.start+1

# Relative risk for all countries and ages the same 
RR.M <- matrix(1.8891, ncol=1, nrow=n.age)
RR.F <- matrix(1.5469, ncol=1, nrow=n.age)     

Prev.name = paste("Prev_ADL_NL",gender,sep="_")
Mort.name = paste("Mort_NL", gender,sep="_")
if (gender=="F") RR_D <- RR.F
if (gender=="M") RR_D <- RR.M
TotRates.1 <- read.table(file = paste(Mort.name,"csv",sep="."),
                     header=T, sep=",", colClasses ="numeric")      
Prev.1 <- read.table(file =  paste(Prev.name,"csv",sep=".")
                   ,header=T, sep=",", colClasses ="numeric")
     
TotRates = TotRates.1[TotRates.1[,1] %in% age.start:age.end,2]
Prev_nD <- matrix(rep(Prev.1[Prev.1[,1] %in% age.start:age.end,2],1), nrow=n.age)
Prev_D <- matrix(rep(Prev.1[Prev.1[,1] %in% age.start:age.end,3],1), nrow=n.age)

############################################################
# 1)   FIRST STEP CALCULATE THE MORTALITY 
#           START VALUES FOR SIMULATENOUS ESTIMATION
###########################################################
m_D = matrix(0,nrow=n.age,ncol=1)
m_nD = matrix(0,nrow=n.age,ncol=1)
rownames(m_D) = age.start:age.end
rownames(m_nD) = age.start:age.end

for (age in age.start:age.end) {
    i = age-age.start+1
    prev = c(Prev_D[i],Prev_nD[i])
    relrisk = RR_D[i]
    r0 = start.rates2(prev,relrisk,TotRates[i])
    r1 = r0 - calratesS.fun(r0,prev,relrisk,TotRates[i])/
              calratesS.dfun(r0,prev,relrisk,TotRates[i])
    while (abs(r0-r1) > 1e-6) {
       r0=r1
       r1 = r0 - calratesS.fun(r0,prev,relrisk,TotRates[i])/
              calratesS.dfun(r0,prev,relrisk,TotRates[i])
    }
    m_D[i]=relrisk*r1 
    m_nD[i]=r1    
}
 

#######################################################################
# 2)   SECOND STEP CALCULATION OF THE INCIDENCE RATES FOR EACH GROUP
#           START VALUES FOR SIMULATENOUS ESTIMATION
# from DISABILITY MODEL, i.e. (age-year specific) Markov model 
# with rate matrix  (conditional on smoking status)
#
#          Dis         non-Dis   death
#  M = -(phi+m_D)       0        m_D
#         theta    -(theta+m_N)  m_N
#          0            0         0
#
#################################################################
theta00= matrix(0,nrow=n.age-1,ncol=1)
rownames(theta00) = age.start:(age.end-1)
theta= matrix(0,nrow=n.age-1,ncol=1)
rownames(theta) = age.start:(age.end-1)
for (age in age.start:(age.end-1)) {
    i = age-age.start+1
    prop0 = c(Prev_D[i],Prev_nD[i])
    prop1 = c(Prev_D[i+1],Prev_nD[i+1])
    mort.rates = c(m_D[i],m_nD[i])
    mDis = mort.rates[1]
    mnDis = mort.rates[2]
    theta0 = start.inc.rates(prop0[2],prop1[2],1)
    theta00[i]=theta0
    theta1 = theta0 - fun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)/
                     dfun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)
    while (abs(theta0-theta1) > 1e-6) {
      theta0 = theta1
      theta1 = theta0 - fun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)/
                       dfun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)
    }
    theta[i]=theta0
}

mnD.mat = matrix(0,nrow= n.age,ncol=1)
rownames(mnD.mat) = age.start:(age.end)
pars.mat = matrix(0,nrow=n.age-1,ncol=1)
rownames(pars.mat) = age.start:(age.end-1)
colnames(pars.mat) = c("Dis_incidence")
for (age in age.start:(age.end-1)) {
    i = age-age.start+1
    mnD.mat[i] = m_nD[i]
    pars.mat[i] = theta[i]
}
     
Surv.mat = matrix(0,nrow=n.age-1,ncol=1)

for (age in age.start:(age.end-1)) {
    i = age-age.start+1
    mnD0 = mnD.mat[i]
    # optimization over mnD
    prop0m = c(Prev_D[i],Prev_nD[i])
    names(prop0m)=c("D","nD")
    relrisk = RR_D[i]
    theta1a = pars.mat[i]
    optm=optimize(f=surv.inc.adl,interval=c(0,1),theta=theta1a,prop0=prop0m,
                  relrisk=relrisk, mtot=TotRates[i],phi=0 ) 
    mnD1 = optm$minimum
    mnD.mat[i] = mnD1
 
    # optimization over theta
    prop0 = c(Prev_D[i],Prev_nD[i])
    prop1 = c(Prev_D[i+1],Prev_nD[i+1])
    mort.rates = c(relrisk*mnD1,mnD1)
    mDis = mort.rates[1]
    mnDis = mort.rates[2]
    theta0=theta[i]
    theta1 = theta0 - fun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)/
                     dfun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)
    while (abs(theta0-theta1) > 1e-6) {
      theta0 = theta1
      theta1 = theta0 - fun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)/
                       dfun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)
    }   
    pars.mat[i]=theta1
    while (abs(mnD0-mnD1) > 1e-10) {
      mnD0 = mnD1
      # optimization over mnSnD
      theta1a = pars.mat[i]
      optm=optimize(f= surv.inc.adl,interval=c(0,1),theta=theta1a,prop0=prop0m,
                   relrisk=relrisk, mtot=TotRates[i], phi=0 ) 
      mnD1 = optm$minimum
        
      # optimization over theta
      mDis = mort.rates[1]
      mnDis = mort.rates[2]
      theta0=theta[i]
      theta1 = theta0 - fun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)/
                     dfun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)
      while (abs(theta0-theta1) > 1e-6) {
         theta0 = theta1
         theta1 = theta0 - fun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)/
                       dfun.inc(theta0, prop0[1], prop1[1], mDis,mnDis,phi=0)
      }   
      pars.mat[i]=theta1
    } 
    mnD.mat[i] = mnD1
    Surv.mat[i]=exp(-TotRates[i])
} 
i = age.end-age.start+1
mnD.mat[i]=m_nD[i]

theta = pars.mat
theta.name = paste("Disinc_ADL_NL", gender, sep="_")
theta.name = paste(theta.name,"txt",sep=".")
write.table(theta,theta.name, sep = ";", quote=F, eol = "\n", col.names=T, row.names=T)

mort.name = paste("Mort_ADL_NL", gender, sep="_")
mort.name = paste(mort.name,"txt",sep=".")
mort_ADL = cbind(mnD.mat,mnD.mat*RR_D)
colnames(mort_ADL) = c("mort_nD", "mort_D")
rownames(mort_ADL) = age.start:age.end
write.table(mort_ADL,mort.name, sep = ";", quote=F, eol = "\n", col.names=T, row.names=T)
